package com.boman.gen.controller;

import com.boman.common.core.web.controller.BaseController;
import com.boman.common.core.web.domain.AjaxResult;
import com.boman.common.redis.service.RedisService;
import com.boman.gen.domain.GenTable;
import com.boman.gen.domain.GenTableColumn;
import com.boman.gen.service.IGenTableColumnService;
import com.boman.gen.service.IGenTableService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author shiqian
 * @description 获取gen_table
 * @date 2021年03月17日 16:44
 **/
@RestController
@RequestMapping("/my")
public class MyController extends BaseController {

    @Autowired
    private IGenTableService genTableService;
    @Autowired
    private IGenTableColumnService genTableColumnService;
    @Autowired
    private RedisService redisService;

    /**
     * 功能描述: 查询代码生成列表，table中封装columnList
     *
     * @param genTable 查询条件
     * @return AjaxResult
     */
    @GetMapping("/loadTable")
    public AjaxResult loadTable(GenTable genTable) throws ClassNotFoundException {
        List<GenTable> tableList = genTableService.selectGenTableList(genTable);
        if (null == tableList || tableList.isEmpty()) {
            return AjaxResult.success();
        }

        List<Long> tableIdList = tableList.stream().map(GenTable::getTableId).collect(Collectors.toList());
        List<GenTableColumn> genTableColumns = genTableColumnService.listByTableIdList(tableIdList);
        if (null == genTableColumns || genTableColumns.isEmpty()) {
            return AjaxResult.success();
        }

        packTableAndInsertToRedis(tableList, genTableColumns);
        return AjaxResult.success(tableList);
    }

    /**
     * 功能描述: 把table中塞入对应的column， 同时把所有的信息塞到redis中
     *
     * @param tableList       tableList
     * @param genTableColumns 所有的列
     */
    private void packTableAndInsertToRedis(List<GenTable> tableList, List<GenTableColumn> genTableColumns) {
        for (GenTable table : tableList) {
            List<GenTableColumn> columnList = new ArrayList<>(16);
            for (GenTableColumn tableColumn : genTableColumns) {
                if (table.getTableId().equals(tableColumn.getTableId())) {
                    columnList.add(tableColumn);
                }
            }

            table.setColumns(columnList);
            redisService.setCacheObject("table:" + table.getTableName(), table, 12L, TimeUnit.DAYS);
        }
    }


    /**手动更新redis缓存表数据
    * @Description
    * @author tjf
    * @Date 2021/3/29
    */
    public AjaxResult updateRedisTable(){
        //获取所有的表和表字段,更新到redis中
        List<GenTable> tableList = genTableService.selectGenTableList(new GenTable());
        if (null == tableList || tableList.isEmpty()) {
            return AjaxResult.error("当前没有可更新数据");
        }
        List<Long> tableIdList = tableList.stream().map(GenTable::getTableId).collect(Collectors.toList());
        List<GenTableColumn> genTableColumns = genTableColumnService.listByTableIdList(tableIdList);
        if (null == genTableColumns || genTableColumns.isEmpty()) {
            return AjaxResult.error("当前没有可更新数据");
        }
        packTableAndInsertToRedis(tableList, genTableColumns);
        return AjaxResult.success();
    }
}
